/*--------------------------------------------------------------------------*/
/*--------------------------- File pds2mngn.C ------------------------------*/
/*--------------------------------------------------------------------------*/
/*--                							  --*/
/*--  Little translator: translate MMCF problems generated by the PDS     --*/
/*--  generator into the 4-files Mnetgen format. Should also work as      --*/
/*--  translator form 3-files to 4-files Mnetgen format, under the        --*/
/*--  assumption that all arcs with the same "name" are listed            --*/
/*--  consecutively in the file.                                          --*/
/*--                                                                      --*/
/*--  One detail of the Pds generator has to be taken into account: the   --*/
/*--  generator produces arcs with the same "name" (those funny strings   --*/
/*--  in the .arc file) but different mutual capacity pointers. This is   --*/
/*--  unfair, and such arcs are mapped by our translator into different   --*/
/*--  arcs with different names, even though the starting and ending      --*/
/*--  nodes are the same.       					  --*/
/*--                                                                      --*/
/*--                            VERSION 1.20                              --*/
/*--                	       15 - 07 - 1999			       	  --*/
/*--                     			                          --*/
/*-- 		     Original Idea and Implementation by:		  --*/
/*--                							  --*/
/*--			       Antonio Frangioni       			  --*/
/*--                							  --*/
/*--   			   Operations Research Group			  --*/
/*--			  Dipartimento di Informatica			  --*/
/*--   			     Universita' di Pisa			  --*/
/*--                							  --*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#include <fstream.h>
#include <assert.h>
#include <string.h>

#define NAMELENGHT 16

/*--------------------------------------------------------------------------*/

static int StrEql( char *s1 , char *s2 )
{
 int i = 0;

 while( ( s1[ i ] != '\0' ) && ( s2[ i ] != '\0' ) &&
        ( s1[ i ] == s2[ i ] ) )
  i++;

 return( s1[ i ] == s2[ i ] );
 }

/*--------------------------------------------------------------------------*/

int main( int argc , char **argv )
{
 if( argc < 3 )
 {
  cerr << "Usage: pds2mngn <input_file_name> <ouput_file_name>" << endl;
  return( 1 );
  }

 int l1 = strlen( argv[ 1 ] );

 char *iName;
 iName = new char[ l1 + 5 ];
 strcpy( iName , argv[ 1 ] );

 int l2 = strlen( argv[ 2 ] );

 char *oName;
 oName = new char[ l2 + 5 ];
 strcpy( oName , argv[ 2 ] );

 ifstream iFile;
 ofstream oFile;

 // part one: node supply info - - - - - - - - - - - - - - - - - - - - - - -

 strcpy( iName + l1 , ".nod" );
 strcpy( oName + l2 , ".sup" );

 iFile.open( iName , ios::in );

 if( ! iFile )
 {
  cerr << "ERROR: opening file " << iName << endl;
  return( 1 );
  }

 oFile.open( oName , ios::out );

 int NumNodi = 0;
 int NumComm = 0;

 int n = 0;

 while( iFile >> n )
 {
  int k;
  float s;

  assert( iFile >> k );
  assert( iFile >> s );

  if( n > NumNodi )
   NumNodi = n;

  if( k > NumComm )
   NumComm = k;

  oFile << n << "\t" << k << "\t" << s << endl;
  }

 iFile.close();
 oFile.close();

 // part two: mutual capacities info - - - - - - - - - - - - - - - - - - - - -

 strcpy( iName + l1 , ".mut" );
 strcpy( oName + l2 , ".mut" );

 iFile.open( iName , ios::in );

 if( ! iFile )
 {
  cerr << "ERROR: opening file " << iName << endl;
  return( 1 );
  }

 oFile.open( oName , ios::out );

 int NumVincoli = 0;

 while( iFile >> n )
 {
  float m;

  assert( iFile >> m );

  if( n > NumVincoli )
   NumVincoli = n;

  oFile << n << "\t" << m << endl;
  }

 iFile.close();
 oFile.close();

 // part three: arcs info - - - - - - - - - - - - - - - - - - - - - - - - - -

 strcpy( iName + l1 , ".arc" );
 strcpy( oName + l2 , ".arc" );

 iFile.open( iName , ios::in );

 if( ! iFile )
 {
  cerr << "ERROR: opening file " << iName << endl;
  return( 1 );
  }

 oFile.open( oName , ios::out );

 int NumArchi = 1;

 char newn[ NAMELENGHT ];
 char oldn[ NAMELENGHT ];

 // the Pds generator uses the same "name" (char string) for arcs with
 // the same (sn, en) but different mutual capacities: those arcs have to
 // be mapped into arcs with different names, and a vector with the mut.
 // cap. pointers is used for the purpose.

 int *tv = new int[ NumComm + 1 ];

 *tv = -1;  // -1 is not a valid mutual capacity pointer, while 0 is

 while( iFile >> newn )
 {
  int sn;
  int en;
  int k;
  float cst;
  float cap;
  int m;
  int i = 0;

  assert( iFile >> sn );
  assert( iFile >> en );
  assert( iFile >> k );
  assert( iFile >> cst );
  assert( iFile >> cap );
  assert( iFile >> m );

  if( ! StrEql( newn , oldn ) )
  {
   // count how many arcs were created last time

   while( tv[ i ] != -1 )
    i++;

   NumArchi += i;

   for( i = NumComm + 1 ; i-- ; )
    tv[ i ] = -1;

   for( i = NAMELENGHT ; i-- ; )
    oldn[ i ] = newn[ i ];

   *tv = m;
   i = NumArchi;
   }
  else
  {
   // find this couple (name, pointer) ...

   while( tv[ i ] != m && tv[ i ] != -1 )
    i++;

   // ... or create it

   tv[ i ] = m;
   i += NumArchi;
   }

  oFile << i << "\t" << sn << "\t" << en << "\t" << k << "\t" << cst << "\t"
        << cap << "\t" << m << endl;
  }

 // count how many arcs were created the very last time

 n = 0;
 while( tv[ n ] != -1 )
  n++;

 // set the total number of arcs

 NumArchi += n;
 NumArchi--;

 delete( tv );

 iFile.close();
 oFile.close();

 // part four: writing summary infos - - - - - - - - - - - - - - - - - - - - -

 strcpy( oName + l2 , ".nod" );
 oFile.open( oName , ios::out );

 oFile << NumComm << endl << NumNodi << endl << NumArchi << endl
       << NumVincoli << endl;

 oFile.close();

 return( 0 );

 }  // end( main )

/*--------------------------------------------------------------------------*/
/*-------------------------- End File pds2mngn.C ---------------------------*/
/*--------------------------------------------------------------------------*/
